/*
**	labels.c
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "opencasl.h"

static LABEL	**labels;
static EXLABEL	**exLabels;
static int		nlabel = 0;
static int		nExLabel = 0;

int initLable()
{
	labels = (LABEL**) calloc(MAXLABEL, sizeof(LABEL*));
	exLabels = (EXLABEL**) calloc(MAXLABEL, sizeof(EXLABEL*));
	return labels != NULL && exLabels != NULL;
}

LABEL *newLable(char *name, word address)
{
	LABEL	*n;

	if ((n = (LABEL*)malloc(sizeof(LABEL))) != NULL) {
		strncpy(n->name, name, MAXNAME);
		n->address = address;
	}
	return n;
}

EXLABEL *newExLabel(char *name, word address, word value)
{
	EXLABEL	*n;

	if ((n = (EXLABEL*)malloc(sizeof(EXLABEL))) != NULL) {
		strncpy(n->name, name, MAXNAME);
		n->address = address;
		n->value = value;
	}
	return n;
}

void clearLabels()
{
	if (0 < nlabel) {
		while (0 <= --nlabel) {
			free(labels[nlabel]);
		}
	}
	if (0 < nExLabel) {
		while (0 <= --nExLabel) {
			free(exLabels[nExLabel]);
		}
	}
}

void deleteLabel()
{
	clearLabels();
	free(labels);
	free(exLabels);
}

int searchLabel(char* name)
{
	int		i;
	int		result = (-1);
	
	for (i = 0; i < nlabel; i++) {
		if (strcmp(name, labels[i]->name) == 0) {
			result = i;
			break;
		}
	}
	return result;
}

int searchExLabel(char *name)
{
	int		i;
	int		result = (-1);
	
	for (i = 0; i < nExLabel; i++) {
		if (strcmp(name, exLabels[i]->name) == 0) {
			result = i;
			break;
		}
	}
	return result;
}

int addLabel(char *name, word address)
{
	int		result = FALSE;

	if (searchLabel(name) < 0 && nlabel < MAXLABEL - 1) {
		if ((labels[nlabel++] = (LABEL*)newLable(name, address)) != NULL) {
			result = TRUE;
		}
	}
	return result;
}

int addExLabel(char *name)
{
	int		result = FALSE;

	if (searchExLabel(name) < 0 && nExLabel < MAXLABEL - 1) {
		if ((exLabels[nExLabel++] = (EXLABEL*)newExLabel(name, 0, 0)) != NULL) {
			result = TRUE;
		}
	}
	return result;
}

int setLabelAddress(char *name, word address)
{
	int		result;
	
	if (0 <= (result = searchLabel(name))) {
		labels[result]->address = address;
	}
	return result;
}

int setExLabelAddress(char *name, word address)
{
	int		result;
	
	if (0 <= (result = searchExLabel(name))) {
		labels[result]->address = address;
	}
	return result;
}

int setExLabelValue(char *name, word value)
{
	int		result;
	
	if (0 <= (result = searchExLabel(name))) {
		exLabels[result]->value = value;
	}
	return result;
}

word getLabelAddress(char *name)
{
	int		p;
	word	result = 0;
	
	if (0 <= (p = searchLabel(name))) {
		result = labels[p]->address;
	}
	if (result == 0) {
		result = getExLabelAddress(name);
	}
	return result;
}

word getExLabelAddress(char *name)
{
	int		p;
	word	result = 0;
	
	if (0 <= (p = searchExLabel(name))) {
		result = exLabels[p]->address;
	}
	return result;
}

word getExLabelValue(char *name)
{
	int		p;
	word	result = 0;
	
	if (0 <= (p = searchExLabel(name))) {
		result = exLabels[p]->value;
	}
	return result;
}

word alignExLabels(word address)
{
	int		i;
	char	buf[BUFSIZ];
	
	for (i = 0; i < nExLabel; i++) {
		sprintf(buf, "TMP%03d", i);
		strcpy(exLabels[i]->alias, buf);
		exLabels[i]->address = address++;
		exLabels[i]->value = getExLabelsNameValue(exLabels[i]->name);
	}
	return address;
}

word getExLabelsNameValue(char *s)
{
	word	result = 0;
	
	if (*s == '=') {
		s++;
		if (*s == '\'') {
			result = *(s + 1);
		} else {
			result = getWord(s);
		}
	}
	return result;
}

void printLabels(FILE *fp)
{
	int		i;
	
	for (i = 0; i < nlabel; i++) {
		fprintf(fp, "%04X: %s\n", labels[i]->address, labels[i]->name);
	}
	for (i = 0; i < nExLabel; i++) {
		fprintf(fp, "%04X: %s\n",
			exLabels[i]->address, exLabels[i]->alias);
	}
}

void printExLabelsPrn(FILE *prn, word address)
{
	int		i;
	
	for (i = 0; i < nExLabel; i++) {
		fprintf(prn, "%04X: %s\t%04X\t; %s\n",
			exLabels[i]->address, exLabels[i]->alias, exLabels[i]->value, exLabels[i]->name);
	}
}

extern	FILE	*obj;

void outExLabelsObj()
{
	int		i;
	word	ope;
	
	for (i = 0; i < nExLabel; i++) {
		ope = exLabels[i]->value;
		fwrite(&ope, sizeof(word), 1, obj);
	}
}


