/*
**	command.c
*/
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "opencomet.h"

static	word	lastDumpPoint = ORGADDR;
static	int		lastDumpSize = 1;

void executeDump(int n, char *items[])
{
	word	adr, mem;
	word	adr1 = 0;
	word	adr2 = 0;
	char	sym[BUFSIZ];

	if (1 < n) {
		adr1 = getOpe1(items[1]);
	}
	if (2 < n) {
		adr2 = adr1 + getOpe1(items[2]);
	}
	if (adr1 == 0) {
		adr1 = lastDumpPoint;
	}
	if (adr2 == 0) {
		adr2 = adr1 + lastDumpSize;
	}
	for (adr = adr1; adr < adr2; adr++) {
		mem = getMem(adr);
		searchLabel(sym, adr);
		printf("%04X:\t%s\t#%04hX ", adr, sym, mem);
		if (isprint(mem)) {
			printf("\'%c\'", mem);
		}
		printf("\n");
	}
	lastDumpPoint = adr2;
	lastDumpSize = adr2 - adr1;
}

static	word	lastListPoint = ORGADDR;
static	int		lastListSize = 16;

void executeList(int n, char *items[])
{
	word	adr;
	word	adr1 = 0;
	word	adr2 = 0;
	char	list[BUFSIZ];

	if (1 < n) {
		adr1 = getOpe1(items[1]);
	}
	if (2 < n) {
		adr2 = adr1 + getOpe1(items[2]);
	}
	if (adr1 == 0) {
		adr1 = lastListPoint;
	}
	if (adr2 == 0) {
		adr2 = adr1 + lastListSize;
	}
	adr = adr1;
	while (adr < adr2) {
		adr += disAssem(list, adr);
		printf("%s\n", list);
	}
	lastListPoint = adr2;
	lastListSize = adr2 - adr1;
}

void executeStep(int n, char *items[])
{
	int		i;
	int		count = 1;
	word	progc;
	
	if (1 < n) {
		count = atoi(items[1]);
	}
	
	for (i = 0; i < count; i++) {
		progc = getPR();
		if (0x0010 <= progc && progc < MEMSIZE) {
			execute(progc);
			if (i < count - 1) {
				printCPU();
				linePrint();
			}
		} else {
			setPR(progc);
			fprintf(stderr,
				"PRの値が範囲外なのでステップ実行できません\n");
			break;
		}
	}
}

void executeSetRegister(int n, char *items[])
{
	int		reg;
	word	value;
	
	if (n == 3) {
		value = getOpe1(items[2]);
		if (isdigit(*items[1])) {
			reg = atoi(items[1]);
			if (0 <= reg && reg <= 7) {
				setReg(reg, value);
			} else {
				fprintf(stderr, "レジスタは 0〜7で指定して下さい\n");
			}
		} else if (strcmp(items[1], "PR") == 0) {
			setPR(value);
		}
	}
}

void executeRun(int n, char *items[])
{
	int		bc;
	word	progc;
	word	stopAddress = 0;

	if (n == 2) {
		stopAddress = getOpe1(items[1]);
	}

	while (TRUE) {
		progc = getPR();
		bc = getBP(progc);
		if (0 < bc) {
			bc--;
			if (bc == 0) {
				break;
			} else {
				setBP(progc, bc);
			}
		}
		if (progc == stopAddress) {
			break;
		}
		if (0x0010 <= progc && progc < MEMSIZE) {
			execute(progc);
		} else {
			setPR(progc);
			fprintf(stderr,
				"PRの値が範囲外なので実行を停止しました\n");
			break;
		}
	}
}

void executeSetBP(int n, char *items[])
{
	int		count = 1;
	word	address;

	if (1 < n) {
		if (2 < n) {
			count = atoi(items[2]);
		}
		address = getOpe1(items[1]);
		setBP(address, count);
	}
}

void executeClearBP(int n, char *items[])
{
	word	address;

	if (1 < n) {
		address = getOpe1(items[1]);
		setBP(address, 0);
	}
}
