/*
**	cpu.c
*/
#include <stdio.h>
#include <stdlib.h>
#include "opencomet.h"

static word		reg[8];
static word		freg = 0;
static word		pr = ORGADDR;
static MEMORY	*mem = NULL;
static word		sp;

void initMemory()
{
	mem = (MEMORY*) calloc(sizeof(MEMORY), MEMSIZE);
}

void initEnvironment()
{
	int		i;

	for (i = 0; i < 8; i++) {
		reg[i] = 0;
	}
	sp = MEMSIZE;
	pr = ORGADDR;
	freg = 0;
}

void clearMemory()
{
	int		i;
	
	if (mem) {
		for (i = 0; i < MEMSIZE; i++) {
			mem[i].mem = 0;
		}
	}
}

void clearBreakPoint()
{
	int		i;
	
	if (mem) {
		for (i = 0; i < MEMSIZE; i++) {
			mem[i].bp = 0;
		}
	}
}

void deleteMemory()
{
	if (mem) {
		free(mem);
		mem = NULL;
	}
}

void setMem(word address, word value)
{
	if (mem) {
		if (0 <= address && address < MEMSIZE) {
			mem[address].mem = value;
		}
	}
}

word getMem(word address)
{
	word	result = (-1);
	
	if (mem) {
		if (0 <= address && address < MEMSIZE) {
			result = mem[address].mem;
		}
	}
	return result;
}

void setBP(word address, int count)
{
	if (mem) {
		if (0 <= address && address < MEMSIZE) {
			mem[address].bp = count;
		}
	}
}

int getBP(word address)
{
	int		result = (-1);
	
	if (mem) {
		if (0 <= address && address < MEMSIZE) {
			result = mem[address].bp;
		}
	}
	return result;	
}

void setReg(int n, word value)
{
	if (0 <= n && n < 8) {
		reg[n] = value;
	}
}

word getReg(int n)
{
	word	result = 0;
	
	if (0 <= n && n < 8) {
		result = reg[n];
	}
	return result;
}

void setPR(word address)
{
	pr = address;
}

word getPR()
{
	return pr;
}

void setFlag(word flag)
{
	freg = flag;
}

word getFlag()
{
	return freg;
}

void pushData(word val)
{
	setMem(--sp, val);
}

word popData()
{
	return getMem(sp++);
}

void printCPU()
{
	int		i;
	
	for (i = 0; i < 4; i++) {
		printf("GR%1d:%04hX ", i, reg[i]);
	}
	printf(" PR:%04hX  ST:%04hX\n", pr, sp);
	for (i = 4; i < 8; i++) {
		printf("GR%1d:%04hX ", i, reg[i]);
	}
	printf(" FLAG(");
	printf((freg & OFLG) ? "OV,": "NO,");
	printf((freg & SFLG) ? "MI,": "PL,");
	printf((freg & ZFLG) ? "ZR)\n": "NZ)\n");
}
