/*
**	labels.c
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "opencomet.h"

static char *getSym(char *s);
static int isSymbol(char *s);
static word atoh(char *s);

static	LABEL	*label = NULL;
static	int		nlabel;

int initLabel()
{
	nlabel = 0;
	label = (LABEL*) calloc(MAXLABEL, sizeof(LABEL));

	return label != NULL;
}

void deleteLabel()
{
	if (label != NULL) {
		free(label);
	}
}

int readLabel(char* fname)
{
	int		i = 0;
	int		addr;
	FILE	*fp;
	char	*p;
	char	buf[BUFSIZ];
	
	if ((fp = fopen(fname, "r")) != NULL) {
		for (i = 0; fgets(buf, BUFSIZ, fp); i++) {
			removeEOL(buf);
			p = getSym(buf);
			sscanf(buf, "%04X", &addr);
			strncpy(label[i].name, p, MAXNAME);
			label[i].address = (word) addr;
		}
		fclose(fp);
	}
	nlabel = i;
	return i;
}

int searchLabel(char *buf, word address)
{
	int		i;
	int		result = 0;

	*buf = NUL;
	for (i = 0; i < nlabel; i++) {
		if (address == label[i].address) {
			strcpy(buf, label[i].name);
			result = 1;
			break;
		}
	}
	return result;
}

word searchAddress(char *buf)
{
	int		i;
	word	result = 0;
	
	for (i = 0; i < nlabel; i++) {
		if (strcmp(buf, label[i].name) == 0) {
			result = label[i].address;
			break;
		}
	}
	return result;
}

void getTarget(char *buf, word address)
{
	char	target[BUFSIZ];
	
	searchLabel(target, address);
	if (*target != NUL) {
		strcpy(buf, target);
	} else {
		sprintf(buf, "#%04hX", address);
	}
}

word getOpe1(char *s)
{
	word	ope1;

	if (isSymbol(s)) {
		ope1 = searchAddress(s);
	} else {
		if (*s == '#') {
			ope1 = atoh(s + 1);
		} else if (isdigit(*s) || *s == '-') {
			ope1 = atoi(s);
		} else {
			ope1 = 0;
		}
	}
	return ope1;
}

static int isSymbol(char *s)
{
	return isalpha(*s);
}

static word atoh(char *s)
{
	int		n;
	
	sscanf(s, "%X", &n);
	return n;
}

static char *getSym(char *s)
{
	while	(*s != NUL) {
		if (*s == ':') {
			*s = NUL;
			s += 2;
			break;
		}
		s++;
	}
	return s;
}
	
