/*
**	mnemonic.c
*/
#include <stdio.h>
#include <string.h>
#include "opencomet.h"

/*
**	opeCode, opeType, opeSize, mnemonic
**
**	opeType 0	GR, adr[, XR] + GR, GR
**			1	GR, adr[, XR]
**			2	adr[, XR]
**			3	GR
**			4	adr
**			5	[adr]
**			6	None
**			7	DS
**			8	DC
**			9	IN, OUT
*/
static	MNEMONIC	mnemo[] = {
	{CMD_NOP,	6, 2, "NOP"},
	{CMD_LD,	0, 2, "LD"},
	{CMD_ST,	1, 2, "ST"},
	{CMD_LAD,	1, 2, "LAD"},
	{CMD_ADDA,	0, 2, "ADDA"},
	{CMD_ADDL,	0, 2, "ADDL"},
	{CMD_SUBA,	0, 2, "SUBA"},
	{CMD_SUBL,	0, 2, "SUBL"},
	{CMD_AND,	0, 2, "AND"},
	{CMD_OR,	0, 2, "OR"},
	{CMD_XOR,	0, 2, "XOR"},
	{CMD_CPA,	0, 2, "CPA"},
	{CMD_CPL,	0, 2, "CPL"},
	{CMD_SLA,	0, 2, "SLA"},
	{CMD_SRA,	0, 2, "SRA"},
	{CMD_SLL,	0, 2, "SLL"},
	{CMD_SRL,	0, 2, "SRL"},
	{CMD_JPL,	2, 2, "JPL"},
	{CMD_JMI,	2, 2, "JMI"},
	{CMD_JNZ,	2, 2, "JNZ"},
	{CMD_JZE,	2, 2, "JZE"},
	{CMD_JOV,	2, 2, "JOV"},
	{CMD_JUMP,	2, 2, "JUMP"},
	{CMD_PUSH,	2, 2, "PUSH"},
	{CMD_POP,	3, 2, "POP"},
	{CMD_CALL,	2, 2, "CALL"},
	{CMD_RET,	6, 2, "RET"},
	{CMD_INPUT,	4, 2, "INPUT"},
	{CMD_OUTPUT,4, 2, "OUTPUT"},
	{CMD_RPUSH, 6, 2, "RPUSH"},
	{CMD_RPOP,  6, 2, "RPOP"},
	{CMD_SVC,	2, 2, "SVC"},
	{CMD_IN,	9, 2, "IN"},
	{CMD_OUT,	9, 2, "OUT"},
	{CMD_EXIT,	6, 2, "EXIT"},
	{CMD_START, 5, 0, "START"},
	{CMD_END,	6, 0, "END"},
	{CMD_DS,	7, 1, "DS"},
	{CMD_DC,	8, 1, "DC"}
};

static int	size_mnemonic = sizeof(mnemo)/sizeof(MNEMONIC);
/*
**	opeType 1	GR, adr[, XR] + GR, GR
**			0	GR, adr[, XR]
**			1	adr[, XR]
**			2	GR
**			3	adr
**			4	[adr]
**			5	None
**			6	DS
**			7	DC
**			8	IN, OUT
*/
int disAssem(char *buf, word address)
{
	int		i;
	int		result = 2;
	int		gr, xr;
	word	mem = getMem(address);
	word	t = mem & 0xff00;
	word	adr;
	char	sym[BUFSIZ];
	char	target[BUFSIZ];
	char	target2[BUFSIZ];

	for (i = 0; i < size_mnemonic; i++) {
		if (mnemo[i].opeCode == t) {
			adr = getMem(address + 1);
			getTarget(target, adr);
			gr  = (mem & 0x00f0) >> 4;
			xr  = mem & 0x000f;
			searchLabel(sym, address);
			switch(mnemo[i].opeType) {
			case 0:
				if (xr & GR2_MASK) {
					xr &= ~GR2_MASK;
					sprintf(buf, "%04X:\t%s\t%s\tGR%1d, GR%1d",
						address, sym, mnemo[i].key, gr, xr);
				} else {
					if (xr == 0) {
						sprintf(buf, "%04X:\t%s\t%s\tGR%1d, %s",
							address, sym, mnemo[i].key, gr, target);
					} else {
						sprintf(buf, "%04X:\t%s\t%s\tGR%1d, %s, GR%1d",
							address, sym, mnemo[i].key, gr, target, xr);
					}
				}
				break;
			case 1:
				if (xr == 0) {
					sprintf(buf, "%04X:\t%s\t%s\tGR%1d, %s",
						address, sym, mnemo[i].key, gr, target);
				} else {
					sprintf(buf, "%04X:\t%s\t%s\tGR%1d, %s, GR%1d",
						address, sym, mnemo[i].key, gr, target, xr);
				}
				break;
			case 2:
				if (xr == 0) {
					sprintf(buf, "%04X:\t%s\t%s\t%s",
						address, sym, mnemo[i].key, target);
				} else {
					sprintf(buf, "%04X:\t%s\t%s\t%s, GR%1d",
						address, sym, mnemo[i].key, target, xr);
				}
				break;
			case 3:
				sprintf(buf, "%04X:\t%s\t%s\tGR%1d",
					address, sym, mnemo[i].key, gr);
				break;
			case 4:
				sprintf(buf, "%04X:\t%s\t%s\t%s",
					address, sym, mnemo[i].key, target);
				break;
			case 5:
				if (adr == 0) {
					sprintf(buf, "%04X:\t%s\t%s\t%s",
						address, sym, mnemo[i].key, target);
				} else {
					sprintf(buf, "%04X:\t%s\t%s",
						address, sym, mnemo[i].key);
				}
				break;
			case 6:
				sprintf(buf, "%04X:\t%s\t%s",
					address, sym, mnemo[i].key);
				break;
			case 8:
				break;
			case 9:
				getTarget(target2, mem & 0x00ff);
				sprintf(buf, "%04X:\t%s\t%s\t%s, %s",
					address, sym, mnemo[i].key, target, target2);
				break;
			default:
				sprintf(buf, "%04X:\t%s\tDC\t%04X, %04X",
					address, sym, mem, adr);
				break;
			}
			if (0 < getBP(address)) {
				strcat(buf, "; BREAKPOINT");
			}
		}
	}
	return result;
}

void linePrint()
{
	char	buf[BUFSIZ];
	
	disAssem(buf, getPR());
	printf("%s\n", buf);
}

word getOpeCode(int pos)
{
	return mnemo[pos].opeCode;
}

int getOpeType(int pos)
{
	return mnemo[pos].opeType;
}

int getOpeSize(int pos)
{
	return mnemo[pos].opeSize;
}
